#!/usr/bin/env python
"""Provides AssetVista datatypes definitions for creating and parsing data.

This module defines classes and methods for creating and parsing AvtypeNumeric,
AvtypeInteger, AvtypeBoolean and AvtypeString.
"""

from __future__ import annotations
import json
from typing import Union
import pint

from .edgeniusFunctions import get_timestamp_edgenius_format


__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.0"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"


def _convert_value_to(value: Union(float, int), origianl_unit: str, target_unit: str) -> Union(float, int):
        ureg = pint.UnitRegistry(preprocessors=[
            lambda s: s.replace('%%', ' permille '),
            lambda s: s.replace('%', ' percent '),
        ])
        ureg.define('permille = 0.001 = %%')  # or 'permille = 1 / 1000 = %%'
        ureg.define('percent = 0.01 = %')

        if ureg(origianl_unit).check('[temperature]'):
            Q_ = ureg.Quantity
            value_with_unit = Q_(value, ureg[origianl_unit])
            return value_with_unit.to(target_unit).magnitude
        else:
            value_with_unit = value * ureg(origianl_unit)
            return value_with_unit.to(target_unit).magnitude


class AvtypeNumeric:
    """Type definition for Numeric (float) values to be used in the AssetVista Edgenius environment.
    
    Args:
        description (str): Description of the variable
        value (float): Variable value. It displays the 'simulation_value' if 'simulation_enabled' is True, otherwise it displays the 'value_from_input' value.
        unit (str): Variable unit (e.g.: current, voltage, meters, etc). (It is recommended to use the definitions from Pint package)
        simulation_enabled (bool): If enabled, the 'simulation_value' property will override the 'value' value.
        simulation_value (float): Value to be ised when 'siulation_enabled' is True
        max (float): The maximum value this variable can indicate. If the 'value' is above 'max', 'out_of_range' is set to True.
        min (float): The minimum value this variable can indicate. If the 'value' is below 'min', 'out_of_range' is set to True.
        value_from_input (float): Value to be assigned to the class. It is displayed in the 'value' if 'simualtion_enabled' is False.
        in_use (bool, optional): Indicates if this variable is in use. Defaults to True.
        timestamp (str, optional): Timestamp last time the variable was calculated. Defaults to Now() (in Edgenius format).
        quality (int, optional): Variable quality. Defaults to 1024.
        dataType (str, optional): String indicating the datatype from avtypes. Defaults to 'AvtypeNumeric'.
    """

    # class attributes are only used for defining the BaseModel attributes
    description: str = ""
    value: float = 0.0
    unit: str = ""
    in_use: bool = True
    timestamp: str = ""
    simulation_enabled: bool = False
    simulation_value: float = 0.0
    max: float = 0.0
    min: float = 0.0
    out_of_range: bool = False
    quality: int = 1024
    dataType: str = 'AvtypeNumeric'
    value_from_input: float = 0.0

    def __init__(self, description: str,
                        unit: str,
                        simulation_enabled: bool,
                        simulation_value: float,
                        max: float,
                        min: float,
                        value_from_input: float,
                        in_use: bool = True,
                        timestamp: str = get_timestamp_edgenius_format(),
                        quality: int = 1024,
                        dataType: str = 'AvtypeNumeric'):

        out_of_range = False
        if (min<max):
            if simulation_enabled:
                out_of_range = (simulation_value > max or simulation_value < min)
            else:
                out_of_range = (value_from_input > max or value_from_input < min)
        
        self.description = description
        self.value = simulation_value if simulation_enabled else value_from_input
        self.unit = unit
        self.in_use = in_use
        self.timestamp = timestamp
        self.simulation_enabled = simulation_enabled
        self.simulation_value = simulation_value
        self.max = max
        self.min = min
        self.out_of_range = out_of_range
        self.quality = quality
        self.dataType = dataType
        self.value_from_input = value_from_input

    def __str__(self):
        return json.dumps(self.__dict__)

    def convert_to(self, target_unit: str) -> AvtypeNumeric:
        conv_obj = AvtypeNumeric(description=self.description,
                                    unit=target_unit,
                                    simulation_enabled=self.simulation_enabled,
                                    simulation_value=_convert_value_to(self.simulation_value, self.unit, target_unit),
                                    max=_convert_value_to(self.max, self.unit, target_unit),
                                    min=_convert_value_to(self.min, self.unit, target_unit),
                                    value_from_input=_convert_value_to(self.value_from_input, self.unit, target_unit),
                                    in_use=self.in_use,
                                    timestamp=self.timestamp,
                                    quality=self.quality)

        return conv_obj


class AvtypeInteger:
    """Type definition for Integer (int) values to be used in the AssetVista Edgenius environment.
    
    Args:
        description (str): Description of the variable
        value (int): Variable value. It displays the 'simulation_value' if 'simulation_enabled' is True, otherwise it displays the 'value_from_input' value.
        unit (str): Variable unit (e.g.: current, voltage, meters, etc). (It is recommended to use the definitions from Pint package)
        simulation_enabled (bool): If enabled, the 'simulation_value' property will override the 'value' value.
        simulation_value (int): Value to be ised when 'siulation_enabled' is True
        max (int): The maximum value this variable can indicate. If the 'value' is above 'max', 'out_of_range' is set to True.
        min (int): The minimum value this variable can indicate. If the 'value' is below 'min', 'out_of_range' is set to True.
        value_from_input (int): Value to be assigned to the class. It is displayed in the 'value' if 'simualtion_enabled' is False.
        in_use (bool, optional): Indicates if this variable is in use. Defaults to True.
        timestamp (str, optional): Timestamp last time the variable was calculated. Defaults to Now() (in Edgenius format).
        quality (int, optional): Variable quality. Defaults to 1024.
        dataType (str, optional): String indicating the datatype from avtypes. Defaults to 'AvtypeInteger'.
    """

    # class attributes are only used for defining the BaseModel attributes
    description: str = ""
    value: int = 0
    unit: str = ""
    in_use: bool = True
    timestamp: str = ""
    simulation_enabled: bool = False
    simulation_value: int = 0
    max: int = 0
    min: int = 0
    out_of_range: bool = False
    quality: int = 1024
    dataType: str = 'AvtypeInteger'
    value_from_input: int = 0

    def __init__(self, description: str,
                        unit: str,
                        simulation_enabled: bool,
                        simulation_value: int,
                        max: int,
                        min: int,
                        value_from_input: int,
                        in_use: bool = True,
                        timestamp: str = get_timestamp_edgenius_format(),
                        quality: int = 1024,
                        dataType: str = 'AvtypeInteger'):
        
        out_of_range = False
        if (min<max):
            if simulation_enabled:
                out_of_range = (simulation_value > max or simulation_value < min)
            else:
                out_of_range = (value_from_input > max or value_from_input < min)

        self.description = description
        self.value = simulation_value if simulation_enabled else value_from_input
        self.unit = unit
        self.in_use = in_use
        self.timestamp = timestamp
        self.simulation_enabled = simulation_enabled
        self.simulation_value = simulation_value
        self.max = max
        self.min = min
        self.out_of_range = out_of_range
        self.quality = quality
        self.dataType = dataType
        self.value_from_input = value_from_input

    def __str__(self):
        return json.dumps(self.__dict__)

    def convert_to(self, target_unit: str) -> AvtypeInteger:
        conv_obj = AvtypeInteger(description=self.description,
                                    unit=target_unit,
                                    simulation_enabled=self.simulation_enabled,
                                    simulation_value=int(_convert_value_to(self.simulation_value, self.unit, target_unit)),
                                    max=int(_convert_value_to(self.max, self.unit, target_unit)),
                                    min=int(_convert_value_to(self.min, self.unit, target_unit)),
                                    value_from_input=int(_convert_value_to(self.value_from_input, self.unit, target_unit)),
                                    in_use=self.in_use,
                                    timestamp=self.timestamp,
                                    quality=self.quality)

        return conv_obj


class AvtypeBoolean:
    """Type definition for Boolean (bool) values to be used in the AssetVista Edgenius environment.
    
    Args:
        description (str): Description of the variable
        value (bool): Variable value. It displays the 'simulation_value' if 'simulation_enabled' is True, otherwise it displays the 'value_from_input' value.
        simulation_enabled (bool): If enabled, the 'simulation_value' property will override the 'value' value.
        simulation_value (bool): Value to be ised when 'siulation_enabled' is True
        value_from_input (bool): Value to be assigned to the class. It is displayed in the 'value' if 'simualtion_enabled' is False.
        in_use (bool, optional): Indicates if this variable is in use. Defaults to True.
        timestamp (str, optional): Timestamp last time the variable was calculated. Defaults to Now() (in Edgenius format).
        quality (int, optional): Variable quality. Defaults to 1024.
        dataType (str, optional): String indicating the datatype from avtypes. Defaults to 'AvtypeBoolean'.
    """

    # class attributes are only used for defining the BaseModel attributes
    description: str = ""
    value: bool = False
    in_use: bool = True
    timestamp: str = ""
    simulation_enabled: bool = False
    simulation_value: bool = False
    quality: int = 1024
    dataType: str = 'AvtypeBoolean'
    value_from_input: bool = False

    def __init__(self, description: str,
                        simulation_enabled: bool,
                        simulation_value: bool,
                        value_from_input: bool,
                        in_use: bool = True,
                        timestamp: str = get_timestamp_edgenius_format(),
                        quality: int = 1024,
                        dataType: str = 'AvtypeBoolean'):
        
        self.description = description
        self.value = simulation_value if simulation_enabled else value_from_input
        self.in_use = in_use
        self.timestamp = timestamp
        self.simulation_enabled = simulation_enabled
        self.simulation_value = simulation_value
        self.quality = quality
        self.dataType = dataType
        self.value_from_input = value_from_input

    def __str__(self):
        return json.dumps(self.__dict__)


class AvtypeString:
    """Type definition for String (str) values to be used in the AssetVista Edgenius environment.
    
    Args:
        description (str): Description of the variable
        value (str): Variable value. It displays the 'simulation_value' if 'simulation_enabled' is True, otherwise it displays the 'value_from_input' value.
        simulation_enabled (bool): If enabled, the 'simulation_value' property will override the 'value' value.
        simulation_value (str): Value to be ised when 'siulation_enabled' is True
        value_from_input (str): Value to be assigned to the class. It is displayed in the 'value' if 'simualtion_enabled' is False.
        in_use (bool, optional): Indicates if this variable is in use. Defaults to True.
        timestamp (str, optional): Timestamp last time the variable was calculated. Defaults to Now() (in Edgenius format).
        quality (int, optional): Variable quality. Defaults to 1024.
        dataType (str, optional): String indicating the datatype from avtypes. Defaults to 'AvtypeBoolean'.
    """

    # class attributes are only used for defining the BaseModel attributes
    description: str = ""
    value: str = ""
    in_use: bool = True
    timestamp: str = ""
    simulation_enabled: bool = False
    simulation_value: str = ""
    quality: int = 1024
    dataType: str = 'AvtypeString'
    value_from_input: str = ""

    def __init__(self, description: str,
                        simulation_enabled: bool,
                        simulation_value: str,
                        value_from_input: str,
                        in_use: bool = True,
                        timestamp: str = get_timestamp_edgenius_format(),
                        quality: int = 1024,
                        dataType: str = 'AvtypeString'):
        
        self.description = description
        self.value = simulation_value if simulation_enabled else value_from_input
        self.in_use = in_use
        self.timestamp = timestamp
        self.simulation_enabled = simulation_enabled
        self.simulation_value = simulation_value
        self.quality = quality
        self.dataType = dataType
        self.value_from_input = value_from_input

    def __str__(self):
        return json.dumps(self.__dict__)




#!/usr/bin/env python
"""Provides AssetVista datatypes definitions for creating and parsing data.

This module defines methods for parsing AvtypeNumeric,
AvtypeInteger, AvtypeBoolean and AvtypeString.
"""

import json

from .avlayerTypes import AvtypeNumeric, AvtypeInteger, AvtypeBoolean, AvtypeString


__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.0"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"


def parse_str_to_avtypenumeric(string_value) -> AvtypeNumeric:
    """Parse json values in a string format to AvtypeNumeric

    Args:
        string_value (str): JSON values dumped as string. String must have the AvtypeNumeric properties as keys (e.g: {'value': 8.0, 'description': 'desc' [...]})

    Returns:
        AvtypeNumeric: AvtypeNumeric object parsed
    """
    # load string as json
    string_obj = json.loads(string_value)
    value = string_obj.get('value')
    if value == None:
        return None

    # Read JSON keys
    description = string_obj.get('description')
    unit = string_obj.get('unit')
    in_use = string_obj.get('in_use')
    timestamp = string_obj.get('timestamp')
    simulation_enabled = string_obj.get('simulation_enabled')
    simulation_value = string_obj.get('simulation_value')
    max = string_obj.get('max')
    min = string_obj.get('min')
    quality = string_obj.get('quality')

    # Create AvtypeNumeric
    avtype_numeric = AvtypeNumeric(description = description,
                                    unit = unit,
                                    in_use = in_use,
                                    timestamp = timestamp,
                                    simulation_enabled = simulation_enabled,
                                    simulation_value = simulation_value,
                                    max = max,
                                    min = min,
                                    quality = quality,
                                    value_from_input = value)

    return avtype_numeric


def parse_str_to_avtypeinteger(string_value) -> AvtypeInteger:
    """Parse json values in a string format to AvtypeInteger

    Args:
        string_value (str): JSON values dumped as string. String must have the AvtypeInteger properties as keys (e.g: {'value': 8, 'description': 'desc' [...]})

    Returns:
        AvtypeInteger: AvtypeInteger object parsed
    """
    # load string as json
    string_obj = json.loads(string_value)
    value = string_obj.get('value')
    if value == None:
        return None

    # Read JSON keys
    description = string_obj.get('description')
    unit = string_obj.get('unit')
    in_use = string_obj.get('in_use')
    timestamp = string_obj.get('timestamp')
    simulation_enabled = string_obj.get('simulation_enabled')
    simulation_value = string_obj.get('simulation_value')
    max = string_obj.get('max')
    min = string_obj.get('min')
    out_of_range = string_obj.get('out_of_range')
    quality = string_obj.get('quality')

    # Create AvtypeInteger
    avtype_integer = AvtypeInteger(description = description,
                                    unit = unit,
                                    in_use = in_use,
                                    timestamp = timestamp,
                                    simulation_enabled = simulation_enabled,
                                    simulation_value = simulation_value,
                                    max = max,
                                    min = min,
                                    quality = quality,
                                    value_from_input = value)

    return avtype_integer


def parse_str_to_avtypeboolean(string_value) -> AvtypeBoolean:
    """Parse json values in a string format to AvtypeBoolean

    Args:
        string_value (str): JSON values dumped as string. String must have the AvtypeBoolean properties as keys (e.g: {'value': False, 'description': 'desc' [...]})

    Returns:
        AvtypeBoolean: AvtypeBoolean object parsed
    """
    # load string as json
    string_obj = json.loads(string_value)
    value = string_obj.get('value')
    if value == None:
        return None

    # Read JSON keys
    description = string_obj.get('description')
    in_use = string_obj.get('in_use')
    timestamp = string_obj.get('timestamp')
    simulation_enabled = string_obj.get('simulation_enabled')
    simulation_value = string_obj.get('simulation_value')
    quality = string_obj.get('quality')

    # Create AvtypeBoolean
    avtype_boolean = AvtypeBoolean(description = description,
                                    in_use = in_use,
                                    timestamp = timestamp,
                                    simulation_enabled = simulation_enabled,
                                    simulation_value = simulation_value,
                                    quality = quality,
                                    value_from_input = value)

    return avtype_boolean


def parse_str_to_avtypestring(string_value) -> AvtypeString:
    """Parse json values in a string format to AvtypeString

    Args:
        string_value (str): JSON values dumped as string. String must have the AvtypeString properties as keys (e.g: {'value': 'test', 'description': 'desc' [...]})

    Returns:
        AvtypeString: AvtypeString object parsed
    """
    # load string as json
    string_obj = json.loads(string_value)
    value = string_obj.get('value')
    if value == None:
        return None

    # Read JSON keys
    description = string_obj.get('description')
    in_use = string_obj.get('in_use')
    timestamp = string_obj.get('timestamp')
    simulation_enabled = string_obj.get('simulation_enabled')
    simulation_value = string_obj.get('simulation_value')
    quality = string_obj.get('quality')

    # Create AvtypeBoolean
    avtype_string = AvtypeString(description = description,
                                    in_use = in_use,
                                    timestamp = timestamp,
                                    simulation_enabled = simulation_enabled,
                                    simulation_value = simulation_value,
                                    quality = quality,
                                    value_from_input = value)

    return avtype_string


"""Provides AssetVista datatypes definitions for creating and parsing data to Edgenius.

This module defines classes and methods for creating and parsing in the Edgenius formats.
"""

from typing import Dict, List, Union
import json

from .edgeniusFunctions import get_timestamp_edgenius_format
from .avlayerTypes import AvtypeNumeric, AvtypeInteger, AvtypeBoolean, AvtypeString
from .avlayerFunctions import parse_str_to_avtypenumeric, parse_str_to_avtypeinteger, parse_str_to_avtypeboolean, parse_str_to_avtypestring


__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.0"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"


class EdgeniusInput:
    def __init__(self, key_name: str, value: Union[str, bool, int, float, List[str], List[bool], List[int], List[float]], quality: str = None, timestamp: str = None):
        self.key_name = key_name
        self.value = value
        self.quality = quality
        self.timestamp = timestamp
        self.avtype = self._parse_value_to_avtype(value)

    def _is_value_an_avtype(self, value: str) -> bool:
        """Checks it the payload is a valid json-like object and if its 'datatype' property is one of the valid Avtypes.

        Args:
            value (str): JSON-like string to be parsed to an Avtype

        Returns:
            bool: True if the value is a valid avtype.
        """

        try:
            obj = json.loads(value)
        except:
            return False

        if obj.get('dataType') != None:
            if (obj['dataType'] == 'AvtypeNumeric' or obj['dataType'] == 'AvtypeInteger' or
                obj['dataType'] == 'AvtypeBoolean' or obj['dataType'] == 'AvtypeString'):
                return True

        return False

    def _parse_value_to_avtype(self, value: str) -> Union[AvtypeBoolean, AvtypeInteger, AvtypeNumeric, AvtypeString]:
        if not self._is_value_an_avtype(value): return None

        avinput = json.loads(value)

        if avinput.get('value') == None: return None              # if 'value' existis

        if avinput['dataType'] == 'AvtypeNumeric':
            return parse_str_to_avtypenumeric(value)
        elif avinput['dataType'] == 'AvtypeInteger':
            return parse_str_to_avtypeinteger(value)
        elif avinput['dataType'] == 'AvtypeBoolean':
            return parse_str_to_avtypeboolean(value)
        elif avinput['dataType'] == 'AvtypeString':
            return parse_str_to_avtypestring(value)

        return None


class EdgeniusInputs:
    """Parses the Edgenius Streaming Calculation Engine inputs to Avtypes

    Args:
        payload_from_edgenius (Dict): Dictionary containing the values received from an Edgenius call.
    """

    def __init__(self, payload_from_edgenius: Dict):
        self.payload = payload_from_edgenius
        self.inputs: Dict[str, EdgeniusInput] = {}
        self._inputs_and_values = {}
        self._get_inputs()


    def _get_inputs(self):
        """Create an avtype-like input for every valid input in the payload."""

        self.inputs.clear()
        for key, value in self.payload.items():
            _quality = None
            _timestamp = None

            if value.get('value') != None:              # if 'value' existis
                if value.get('quality') != None: _quality = value['quality']
                if value.get('timestamp') != None: _timestamp = value['timestamp']
                self.inputs[key] = EdgeniusInput(key, value['value'], _quality, _timestamp)
                self._inputs_and_values[key] = value['value']


    def get_input_raw_value(self, key_name: str) -> Union[bool, int, float, str, None]:
        _value = None
        if self.inputs.get(key_name) != None:
            _value = self.inputs[key_name].value

        return _value

    
    def get_input_as_avtype(self, key_name: str) -> Union[AvtypeNumeric, AvtypeInteger, AvtypeBoolean, AvtypeString, None]:
        _value = None
        if self.inputs.get(key_name) != None:
            _value = self.inputs[key_name].avtype

        return _value


    def get_dict_of_inputs_and_values(self, skip_inputs_names: List[str] = None):
        new_dict = dict(self._inputs_and_values)
        if (skip_inputs_names):
            for name in skip_inputs_names:
                new_dict.pop(name)
        return new_dict



class EdgeniusOutput:
    """Write a key vaue in a Edgenius output format
    
    Args:
        key_name (str): Name of the key (variable)
        value ([float, int, bool, str]): Value of the variable
        timestamp (str, optional): Timestamp of the variable. Defaults to None.
        quality (int, optional): Quality. Defaults to None.
    """

    def __init__(self, key_name: str, value, timestamp: str = None, quality: int  = None):
        if not timestamp:
            self.timestamp = get_timestamp_edgenius_format()
        else: 
            self.timestamp = timestamp

        if not quality:
            self.quality = 1024
        else:
            self.quality = quality

        self.key_name = key_name

        if isinstance(value, (AvtypeNumeric, AvtypeInteger, AvtypeBoolean, AvtypeString)):
            self.value = str(value)
        else:
            self.value = value

    def get_key_and_values(self):
        """Return the inputed key and a dictionary in a Edgenius format

        Returns:
            key, value[Dict]: Key and dictionary in a Edgenius format
        """
        values = {"value": self.value, "quality": self.quality, "timestamp": self.timestamp}
        return self.key_name, values


class EdgeniusOutputList:
    """Defines a list of EdgeniusOutputs

    Args:
        list_of_edgenius_outputs (List[EdgeniusOutput]): List of many EdgeniusOutputs
    """

    def __init__(self, list_of_edgenius_outputs: List[EdgeniusOutput]):
        self.outputs = list_of_edgenius_outputs

    def get_all_outputs(self) -> dict:
        """Create and return a dict of EdgeniusOutputs

        Returns:
            dict: Dictionary containing all the EdgeniusOutputs
        """
        all_outputs = {}
        for output in self.outputs:
            output_key, output_value = output.get_key_and_values()
            all_outputs[output_key] = output_value

        return all_outputs


#!/usr/bin/env python
"""Defines useful functions to perform calculations used in the Edgenius environment.

This module defines methods to be used in the Edgenius interfaces.
"""

from datetime import datetime


__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.0"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"


def get_timestamp_edgenius_format(dt: datetime = None) -> str:
    """Get a datetime variable and returns it as string in the Edgenius format.

    Args:
        dt (datetime, optional): Datetime variable to be converted. If this argument is not passed, the function will return the Now() datetime. Defaults to None.

    Returns:
        str: Datetime as string in the edgenius format.
    """
    
    if dt == None:
        dt = datetime.now()
    date = dt.strftime('%Y-%m-%dT%H:%M:%S')
    ms = dt.strftime('.%f')[0:4]
    zone = dt.strftime('Z%Z')
    return date+ms+zone


import os
import pathlib
from typing import Union, List
from fastapi import FastAPI

__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.0"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"

# TODO: Add comments to classes and functions
# TODO: Create test script
# TODO: Replace prints by logs

AMS_FOLDER_NAME = 'assetmonitors'
REQUIRED_FILES_IN_AM_FOLDER = ['_am_def.py', '_cond_def.py', 'router.py']


def _is_valid_am_folder(folder: Union[str, pathlib.Path]):
    ''' Verify if the AM folder contains the minimum files requirements to be an Asset Monitor folder
    '''
    required_files = REQUIRED_FILES_IN_AM_FOLDER.copy()

    if not folder.is_dir(): print(''.join(['Asset Monitor path (', folder.name, ') is not a folder'])); return False

    for subitem in os.listdir(folder):
        if os.path.isfile(os.path.join(folder, subitem)):
            if subitem in required_files:
                required_files.remove(subitem)
                print(''.join(['Requirement ', subitem, ' found on Asset Monitor folder']))

    if len(required_files) == 0:
        print(''.join(['All required files found on ', folder.name]))
        return True
    else:
        print(''.join(['Missing required files on folder ', folder.name]))
        print('Missing files:')
        print(required_files)
        return False



def _get_ams_list(assetmonitors_folder_path, folders_to_be_ignored = []):
    subfolders = []
    for folder in os.scandir(assetmonitors_folder_path):
        if not folder.is_dir(): print(''.join(['Asset Monitor path (', folder.name, ') is not a folder'])); continue
        if folder.name in folders_to_be_ignored: print(''.join(['Folder ', folder.name, ' marked to be ignored.'])); continue
        if folder.name.startswith('__'): print(''.join(['Folder ', folder.name, ' ignored because it starts with dunder (__)'])); continue
        if folder.name.startswith('.'): print(''.join(['Folder ', folder.name, ' ignored because it starts with dor (.)'])); continue
        if not _is_valid_am_folder(folder): print('Invalid AM folder!')

        subfolders.append(folder.name)

    return subfolders


def _include_am_routers(fastapi_app: FastAPI, ams_list):

    # Importing AMs libs
    for am in ams_list:
        import_str = ''.join(['from ', AMS_FOLDER_NAME, '.', am, ' import router as ', am])
        print(''.join(['Importing: ', import_str]))
        exec(import_str)

    # Including routers
    for am in ams_list:
        router = eval(''.join([am, '.router']))
        fastapi_app.include_router(router)

    return fastapi_app


def add_assetvista_routes(fastapi_app: FastAPI, assetmonitors_folder_path, folders_to_be_ignored = []):
    ams_list = _get_ams_list(assetmonitors_folder_path, folders_to_be_ignored)
    return _include_am_routers(fastapi_app, ams_list)


def create_route_example(avtype_numerics: List[str] = None,
                        avtype_integers: List[str] = None,
                        avtype_strings: List[str] = None,
                        avtype_booleans: List[str] = None,
                        numerics: List[str] = None,
                        integers: List[str] = None,
                        strings: List[str] = None,
                        booleans: List[str] = None):

    route_example = {}

    # TODO: Get examples from the avtypes instead of typing each example.

    if avtype_numerics != None:
        for avtype_numeric in avtype_numerics:
            route_example[avtype_numeric] = {'value':
                                                '{"description": "", "value": 40.0, "unit": "", "in_use": true, "timestamp": "2022-05-03T05:24:38.712Z", "simulation_enabled": false, "simulation_value": 200.0, "max": 13.0, "min": 16.0, "out_of_range": false, "quality": 1024, "dataType": "AvtypeNumeric"}',
                                            'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'String'}

    if avtype_integers != None:
        for avtype_integer in avtype_integers:
            route_example[avtype_integer] = {'value':
                                                '{"description": "", "value": 40, "unit": "", "in_use": true, "timestamp": "2022-05-03T05:24:38.712Z", "simulation_enabled": false, "simulation_value": 200, "max": 13, "min": 16, "out_of_range": false, "quality": 1024, "dataType": "AvtypeInteger"}',
                                            'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'String'}

    if avtype_strings != None:
        for avtype_string in avtype_strings:
            route_example[avtype_string] = {'value':
                                                '{"description": "", "value": "test", "in_use": true, "timestamp": "2022-05-03T05:24:38.712Z", "simulation_enabled": false, "simulation_value": "sim value", "quality": 1024, "dataType": "AvtypeString"}',
                                            'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'String'}

    if avtype_booleans != None:
        for avtype_boolean in avtype_booleans:
            route_example[avtype_boolean] = {'value':
                                                '{"description": "", "value": true, "in_use": true, "timestamp": "2022-05-03T05:24:38.712Z", "simulation_enabled": false, "simulation_value": false, "quality": 1024, "dataType": "AvtypeBoolean"}',
                                            'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'String'}

    if numerics != None:
        for numeric in numerics:
            route_example[numeric] = {'value':
                                        40.0,
                                    'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'Numeric'}

    if integers != None:
        for integer in integers:
            route_example[integer] = {'value':
                                        40,
                                    'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'Integer'}

    if strings != None:
        for string in strings:
            route_example[string] = {'value':
                                        'test',
                                    'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'String'}

    if booleans != None:
        for boolean in booleans:
            route_example[boolean] = {'value':
                                        True,
                                    'quality': 1024, 'timestamp': '2021-04-09T16:48:23.3Z', 'dataType': 'Boolean'}

    route_example["previousAlarms"] = {}


    return route_example


import setuptools

install_requires=[]
install_requires.append('pint')

setuptools.setup(
    name="abb_assetvista_avtypes",
    version="0.1.8",
    author="ABB AssetVista Team",
    author_email="BR-assetvista@abb.com",
    description="Package defining the classes and methods required for using the ABB AssetVista library on the Genix Streaming Calculation Engine.",
    url = "https://codebits.abb.com/assetvista/avtypes",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
    python_requires='>=3.6',
    install_requires=install_requires
)


#!/usr/bin/env python
"""Provides AssetVista datatypes definitions for creating logic Conditions and SubConditions.

This module defines the types for avlogic Conditions and SubConditions
"""

from typing import Callable, Dict

from .avlayerTypes import *
from .edgeniusTypes import EdgeniusOutput, EdgeniusOutputList

__author__ = "Tiago Prata"
__copyright__ = "Copyright 2022, ABB Automacao Ltda (Brazil)"
__credits__ = ["ABB Automacao Ltda", "Tiago Prata"]
__license__ = "Copyright"
__version__ = "1.0.1"
__maintainer__ = "AssetVista Team"
__email__ = "BR-assetvista@abb.com"
__status__ = "Development (do not use in production)"

# TODO: Add comments to classes and functions
# TODO: Create test script

class SubCondition:
    def __init__(self, variables, output_value: int, disable_if_any_variable_is_out_of_range: bool = True):
        self.variables = variables
        self.output_value = output_value
        self.disable_if_any_variable_is_out_of_range = disable_if_any_variable_is_out_of_range
        self.function: Callable[..., bool] = None
        self.enabled = self._is_subcond_enabled()


    def _is_subcond_enabled(self):
        enabled = True
        for variable in self.variables:
            if isinstance(variable, (AvtypeNumeric, AvtypeInteger, AvtypeBoolean, AvtypeString)):
                if (not variable.in_use) or variable.out_of_range:
                    enabled = False
                    break
        
        return enabled


class Condition:
    def __init__(self, outputRecords):
        self.subconditions: Dict[int, SubCondition] = {}
        self.active_subcond = self._get_active_subcond_index()
        self.outputRecords = outputRecords

    
    def _get_active_subcond_index(self):
        index = 0
        for key, subcond in self.subconditions.items():
            if subcond.function != None:
                if subcond.enabled and subcond.function():
                    if (index==0) or (key<index):
                        index = key

        return index


    def output_value(self) -> int:
        active_index = self._get_active_subcond_index()
        if active_index == 0: return 0

        return self.subconditions[active_index].output_value


    def are_all_subconditions_disabled(self) -> bool:
        counter = 0
        all_subconditions_disabled = False

        for _, subcond in self.subconditions.items():
            if not subcond.enabled:
                counter += 1

        if len(self.subconditions) == counter:
            all_subconditions_disabled = True

        return all_subconditions_disabled

    
    def conv_outputRecords_to_list_of_edgeniusOutput(self):
        output_list = []
        for item in vars(self.outputRecords):
            output_name = item
            output_value = getattr(self.outputRecords, item)

            output_list.append(EdgeniusOutput(output_name, output_value))
        
        return output_list

    
    def get_edgenius_outputs(self):
        condition_status = EdgeniusOutput('ConditionStatus', self.output_value())
        enabled = EdgeniusOutput('Enabled', not self.are_all_subconditions_disabled())

        outputRecords_list = self.conv_outputRecords_to_list_of_edgeniusOutput()

        outputs = EdgeniusOutputList([condition_status, enabled] + outputRecords_list)

        return outputs.get_all_outputs()